/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JDBCConnectionsModelLocator.as 18499 2009-09-03 14:25:10Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.classloader.model {

import com.adobe.cairngorm.control.CairngormEvent;
import com.adobe.cairngorm.model.ModelLocator;

import mx.collections.ArrayCollection;

import mx.utils.UIDUtil;

import org.ow2.jonas.admin.classloader.business.GetClassLoaderApplicationReportDelegate;
import org.ow2.jonas.admin.classloader.business.GetExportedPackagesDelegate;
import org.ow2.jonas.admin.classloader.business.GetObjectNameGetResourcesDelegate;
import org.ow2.jonas.admin.classloader.business.GetObjectNameLoadClassDelegate;
import org.ow2.jonas.admin.classloader.business.GetStandaloneApplicationsDelegate;
import org.ow2.jonas.admin.classloader.business.GetSystemResourcesDelegate;
import org.ow2.jonas.admin.classloader.business.IGetClassLoaderApplicationReportDelegate;
import org.ow2.jonas.admin.classloader.business.IGetExportedPackagesDelegate;
import org.ow2.jonas.admin.classloader.business.IGetObjectNameGetResourcesDelegate;
import org.ow2.jonas.admin.classloader.business.IGetObjectNameLoadClassDelegate;
import org.ow2.jonas.admin.classloader.business.IGetStandaloneApplicationsDelegate;
import org.ow2.jonas.admin.classloader.business.IGetSystemResourcesDelegate;
import org.ow2.jonas.admin.classloader.view.ClassLoaderView;
import org.ow2.jonas.admin.classloader.view.NodeVisualGraph;


/**
 * Cairngorm Model Locator for the ClassLoader module
 * @author Florent Benoit
 */
[Bindable]
public class ClassLoaderModelLocator implements ModelLocator {

    /**
     * The unique ID of this component
     *
     * @internal
     *   Used to prevent a Cairngorm issue: when a command event is dispatched,
     * every controller that registered this event type receives it, even if
     * located in another module. To prevent this from happening and triggering
     * multiple severe unexpected concurrence bugs, each event dispatched is
     * postfixed with this unique ID.
     */
    public var componentID : String = UIDUtil.createUID();

    /**
     * Singleton instance of this locator
     */
    private static var unique : ClassLoaderModelLocator;


    /**
     * Link to the view.
     */
    private var classLoaderView : ClassLoaderView = null;

	/**
	 * XML Content for the report on classloader for a given application
	 */
	private var xmlClassLoaderInfo : XML = null;

	/**
	 * XML Content for searching packages/classes
	 */
	private var xmlSearchResultPackagesClasses : XML = null;

	/**
	 * Array result for searching system resources
	 */
	private var arraySearchResultSystemResources : Array = null;


	/**
	 * If nothing has been found during the search
	 */
	private var nothingFoundDuringSearch : Boolean = false;


	/**
	 * ObjectName of the selected classLoader.
	 */
	public var selectedClassLoaderObjectName : String = null;


	/**
	 * XML Content for searching classes
	 */
	public var xmlClassLoaderObjectNameLoadClassResult : XML = null;

	/**
	 * Array result for searching system resources
	 */
	public var arrayClassLoaderObjectNameGetResourcesResult : Array = null;


	/**
	 * Delegate for get exported packages.
	 */
	private var getExportedPackagesDelegate : IGetExportedPackagesDelegate = null;

	/**
	 * Delegate for get resources packages.
	 */
	private var getSystemResourcesDelegate : IGetSystemResourcesDelegate = null;

	/**
	 * Delegate for get standalone applications.
	 */
	private var getStandaloneApplicationsDelegate : IGetStandaloneApplicationsDelegate = null;

	/**
	 * Delegate for get classloader report on applications.
	 */
	private var getClassLoaderApplicationReportDelegate : IGetClassLoaderApplicationReportDelegate = null;

	/**
	 * Delegate for loading class on an objectName.
	 */
	private var getObjectNameLoadClassDelegate : IGetObjectNameLoadClassDelegate = null;

	/**
	 * Delegate for loading class on an objectName.
	 */
	private var getObjectNameGetResourcesDelegate : IGetObjectNameGetResourcesDelegate = null;


	/**
	 * Visual Graph component.
	 */
	public var vgraph : NodeVisualGraph = null;

	/**
	 * Showing Graph ?
	 */
	public var showingGraph : Boolean = false;

	/**
	 * Selected XML fragment
	 */
	public var xmlSelectedClassLoader : XML = null;


	/**
	 * Information on all the Applications found on the server.
	 */
	[ArrayElementType("org.ow2.jonas.admin.classloader.vo.ApplicationInfoVO")]
	public var standaloneApplications : ArrayCollection;

	public const CLASS_SEARCH_MODE : String = "Class";
	public const PACKAGE_SEARCH_MODE : String = "Package";
	public const RESOURCE_SEARCH_MODE : String = "Resource";

	/**
	 * SearchMode.
	 */
	private var searchModeValue : String = CLASS_SEARCH_MODE;


	public var searchOK : Boolean = false;

    /**
     * ClassLoaderModelLocator singleton instance getter
     */
    public static function getInstance() : ClassLoaderModelLocator {
        if (unique == null) {

            // Instanciate the ModelLocator
            unique = new ClassLoaderModelLocator();
        }
        return unique;
    }

    /**
     * Constructor
     */
    public function ClassLoaderModelLocator() {
        // Instanciate the singleton
        if (unique != null) {
            throw new Error("Error: Singletons can only be" + "instantiated via getInstance() method!");
        }
		ClassLoaderModelLocator.unique = this;


        // Delegates
	    this.getExportedPackagesDelegate = new GetExportedPackagesDelegate();
		this.getSystemResourcesDelegate = new GetSystemResourcesDelegate();
		this.getStandaloneApplicationsDelegate = new GetStandaloneApplicationsDelegate();
		this.getClassLoaderApplicationReportDelegate = new GetClassLoaderApplicationReportDelegate();
		this.getObjectNameLoadClassDelegate = new GetObjectNameLoadClassDelegate();
		this.getObjectNameGetResourcesDelegate = new GetObjectNameGetResourcesDelegate();
    }


	public function get searchMode() : String {
		return searchModeValue;
	}


	public function set searchMode(searchModeValue : String) : void {
		this.searchModeValue = searchModeValue;
	}

	public function set view(classLoaderView : ClassLoaderView) : void {
		this.classLoaderView = classLoaderView;
	}

	public function set xmlPackages(xmlSearchResultPackagesClasses : XML) : void {
		this.xmlSearchResultPackagesClasses = xmlSearchResultPackagesClasses;
	}

	public function get xmlPackages() : XML {
		return xmlSearchResultPackagesClasses;
	}


	public function set xmlClassLoaderInfoReport(xmlClassLoaderInfo : XML) : void {
		this.xmlClassLoaderInfo = xmlClassLoaderInfo;
	}

	public function get xmlClassLoaderInfoReport() : XML {
		return xmlClassLoaderInfo;
	}



	public function set systemResources(arraySearchResultSystemResources : Array) : void {
		this.arraySearchResultSystemResources = arraySearchResultSystemResources;
	}

	public function get systemResources() : Array {
		return arraySearchResultSystemResources;
	}


	public function dispatch(event : CairngormEvent) : void {
		classLoaderView.dispatchEvent(event);

	}

	public function get exportedPackagesDelegate() : IGetExportedPackagesDelegate {
		return getExportedPackagesDelegate;
	}

	public function get systemResourcesDelegate() : IGetSystemResourcesDelegate {
		return getSystemResourcesDelegate;
	}

	public function get standaloneApplicationsDelegate() : IGetStandaloneApplicationsDelegate {
		return getStandaloneApplicationsDelegate;
	}

	public function get classLoaderApplicationReportDelegate() : IGetClassLoaderApplicationReportDelegate {
		return getClassLoaderApplicationReportDelegate;
	}

	public function get objectNameLoadClassDelegate() : IGetObjectNameLoadClassDelegate {
		return getObjectNameLoadClassDelegate;
	}

	public function get objectNameGetResourcesDelegate() : IGetObjectNameGetResourcesDelegate {
		return getObjectNameGetResourcesDelegate;
	}



	public function set nothingHasBeenFoundDuringSearch(nothingFoundDuringSearch : Boolean) : void {
		this.nothingFoundDuringSearch = nothingFoundDuringSearch;
	}

	public function get nothingHasBeenFoundDuringSearch() : Boolean {
		return nothingFoundDuringSearch;
	}

}
}
